package mmm.deco;


import java.util.List;

import mmm.Mmm;
import mmm.utils.I_URecipeRegistrar;
import mmm.utils.UMaths;
import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Mirror;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;



public class DBlockTable
		extends Block
		implements I_URecipeRegistrar
{

	public final static String ID = "deco_table_";
	public final static String NAME = Mmm.PREFIX + DBlockTable.ID;

	protected static final AxisAlignedBB COLLISION_TOP = UMaths.makeBlockAABB( 0 , 12 , 0 , 16 , 16 , 16 );
	protected static final AxisAlignedBB COLLISION_LEGS[] = { //
			UMaths.makeBlockAABB( 1 , 0 , 1 , 3 , 12 , 3 ) , //
			UMaths.makeBlockAABB( 13 , 0 , 1 , 15 , 12 , 3 ) , //
			UMaths.makeBlockAABB( 13 , 0 , 13 , 15 , 12 , 15 ) , //
			UMaths.makeBlockAABB( 1 , 0 , 13 , 3 , 12 , 15 ), //
	};

	public static final PropertyBool NORTH = PropertyBool.create( "north" );
	public static final PropertyBool EAST = PropertyBool.create( "east" );
	public static final PropertyBool SOUTH = PropertyBool.create( "south" );
	public static final PropertyBool WEST = PropertyBool.create( "west" );
	public static final PropertyBool NW = PropertyBool.create( "nw" );
	public static final PropertyBool NE = PropertyBool.create( "ne" );
	public static final PropertyBool SW = PropertyBool.create( "sw" );
	public static final PropertyBool SE = PropertyBool.create( "se" );
	private static final PropertyBool[] DIRECTIONS = { //
			DBlockTable.NORTH , DBlockTable.NE , DBlockTable.EAST , DBlockTable.SE , DBlockTable.SOUTH ,
			DBlockTable.SW , DBlockTable.WEST , DBlockTable.NW//
	};

	public final E_DWoodType type;


	public DBlockTable( final E_DWoodType type )
	{
		super( Material.WOOD , type.mapColor );
		this.type = type;

		this.setDefaultState( this.blockState.getBaseState( )//
				.withProperty( DBlockTable.NORTH , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.EAST , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.SOUTH , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.WEST , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.NW , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.NE , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.SW , Boolean.valueOf( false ) )//
				.withProperty( DBlockTable.SE , Boolean.valueOf( false ) ) );

		this.setCreativeTab( CreativeTabs.DECORATIONS );
		this.setRegistryName( DBlockTable.ID + type.suffix );
		this.setUnlocalizedName( DBlockTable.NAME + type.suffix );

		this.lightOpacity = 0;
		this.translucent = false;
		this.fullBlock = false;
		this.blockHardness = 2.5f;
		this.blockResistance = 12.5f;
		this.blockSoundType = SoundType.LADDER;
		this.enableStats = false;

		this.setHarvestLevel( "axe" , 0 );
	}


	@Override
	public void registerRecipe( )
	{
		GameRegistry.addShapedRecipe( new ItemStack( this ) , //
				"BBB" , //
				"S S" , //
				'B' , new ItemStack( this.type.block , 1 , this.type.metaData ) , //
				'S' , Items.STICK //
		);
	}


	@Override
	public boolean isOpaqueCube( final IBlockState state )
	{
		return false;
	}


	@Override
	public boolean isFullCube( final IBlockState state )
	{
		return false;
	}


	public boolean canConnectTo( final IBlockAccess worldIn , final BlockPos pos )
	{
		return worldIn.getBlockState( pos ).getBlock( ) == this;
	}


	@Override
	protected BlockStateContainer createBlockState( )
	{
		return new BlockStateContainer( this , new IProperty[] {
				DBlockTable.NORTH , DBlockTable.EAST , DBlockTable.WEST , DBlockTable.SOUTH , DBlockTable.NE ,
				DBlockTable.NW , DBlockTable.SE , DBlockTable.SW
		} );
	}


	@Override
	public int getMetaFromState( final IBlockState state )
	{
		return 0;
	}


	@Override
	public IBlockState getActualState( final IBlockState state , final IBlockAccess worldIn , final BlockPos pos )
	{
		final BlockPos n = pos.north( );
		final BlockPos s = pos.south( );
		final BlockPos w = pos.west( );
		final BlockPos e = pos.east( );
		return state.withProperty( DBlockTable.NORTH , Boolean.valueOf( this.canConnectTo( worldIn , n ) ) )//
				.withProperty( DBlockTable.EAST , Boolean.valueOf( this.canConnectTo( worldIn , e ) ) )//
				.withProperty( DBlockTable.SOUTH , Boolean.valueOf( this.canConnectTo( worldIn , s ) ) )//
				.withProperty( DBlockTable.WEST , Boolean.valueOf( this.canConnectTo( worldIn , w ) ) )//
				.withProperty( DBlockTable.NW , Boolean.valueOf( this.canConnectTo( worldIn , n.west( ) ) ) )//
				.withProperty( DBlockTable.NE , Boolean.valueOf( this.canConnectTo( worldIn , n.east( ) ) ) )//
				.withProperty( DBlockTable.SW , Boolean.valueOf( this.canConnectTo( worldIn , s.west( ) ) ) )//
				.withProperty( DBlockTable.SE , Boolean.valueOf( this.canConnectTo( worldIn , s.east( ) ) ) )//
		;
	}


	@Override
	public IBlockState withRotation( final IBlockState state , final Rotation rot )
	{
		switch ( rot ) {
			case CLOCKWISE_180:
				return state.withProperty( DBlockTable.NORTH , state.getValue( DBlockTable.SOUTH ) )//
						.withProperty( DBlockTable.EAST , state.getValue( DBlockTable.WEST ) )//
						.withProperty( DBlockTable.SOUTH , state.getValue( DBlockTable.NORTH ) )//
						.withProperty( DBlockTable.WEST , state.getValue( DBlockTable.EAST ) )//
						.withProperty( DBlockTable.NW , state.getValue( DBlockTable.SE ) )//
						.withProperty( DBlockTable.NE , state.getValue( DBlockTable.SW ) )//
						.withProperty( DBlockTable.SE , state.getValue( DBlockTable.NW ) )//
						.withProperty( DBlockTable.SW , state.getValue( DBlockTable.NE ) )//
				;
			case COUNTERCLOCKWISE_90:
				return state.withProperty( DBlockTable.NORTH , state.getValue( DBlockTable.EAST ) )//
						.withProperty( DBlockTable.EAST , state.getValue( DBlockTable.SOUTH ) )//
						.withProperty( DBlockTable.SOUTH , state.getValue( DBlockTable.WEST ) )//
						.withProperty( DBlockTable.WEST , state.getValue( DBlockTable.NORTH ) )//
						.withProperty( DBlockTable.NW , state.getValue( DBlockTable.NE ) )//
						.withProperty( DBlockTable.NE , state.getValue( DBlockTable.SE ) )//
						.withProperty( DBlockTable.SE , state.getValue( DBlockTable.SW ) )//
						.withProperty( DBlockTable.SW , state.getValue( DBlockTable.NW ) )//
				;
			case CLOCKWISE_90:
				return state.withProperty( DBlockTable.NORTH , state.getValue( DBlockTable.WEST ) )//
						.withProperty( DBlockTable.EAST , state.getValue( DBlockTable.NORTH ) )//
						.withProperty( DBlockTable.SOUTH , state.getValue( DBlockTable.EAST ) )//
						.withProperty( DBlockTable.WEST , state.getValue( DBlockTable.SOUTH ) )//
						.withProperty( DBlockTable.NW , state.getValue( DBlockTable.SW ) )//
						.withProperty( DBlockTable.NE , state.getValue( DBlockTable.NW ) )//
						.withProperty( DBlockTable.SE , state.getValue( DBlockTable.NE ) )//
						.withProperty( DBlockTable.SW , state.getValue( DBlockTable.SE ) )//
				;
			default:
				return state;
		}
	}


	@Override
	public IBlockState withMirror( final IBlockState state , final Mirror mirrorIn )
	{
		switch ( mirrorIn ) {
			case LEFT_RIGHT:
				return state.withProperty( DBlockTable.NORTH , state.getValue( DBlockTable.SOUTH ) )//
						.withProperty( DBlockTable.SOUTH , state.getValue( DBlockTable.NORTH ) )//
						.withProperty( DBlockTable.NW , state.getValue( DBlockTable.SW ) )//
						.withProperty( DBlockTable.NE , state.getValue( DBlockTable.SE ) )//
						.withProperty( DBlockTable.SW , state.getValue( DBlockTable.NW ) )//
						.withProperty( DBlockTable.SE , state.getValue( DBlockTable.NE ) )//
				;
			case FRONT_BACK:
				return state.withProperty( DBlockTable.EAST , state.getValue( DBlockTable.WEST ) )//
						.withProperty( DBlockTable.WEST , state.getValue( DBlockTable.EAST ) )//
						.withProperty( DBlockTable.NW , state.getValue( DBlockTable.NE ) )//
						.withProperty( DBlockTable.NE , state.getValue( DBlockTable.NW ) )//
						.withProperty( DBlockTable.SW , state.getValue( DBlockTable.SE ) )//
						.withProperty( DBlockTable.SE , state.getValue( DBlockTable.SW ) )//
				;
			default:
				return state;
		}
	}


	@Override
	public AxisAlignedBB getBoundingBox( final IBlockState state , final IBlockAccess source , final BlockPos pos )
	{
		final IBlockState actual = this.getActualState( state , source , pos );
		for ( int i = 0 , dir = 0 ; i < 4 ; i++ , dir += 2 ) {
			final boolean c0 = actual.getValue( DBlockTable.DIRECTIONS[ dir ] );
			final boolean c1 = actual.getValue( DBlockTable.DIRECTIONS[ ( dir + 6 ) % 8 ] );
			final boolean c10 = actual.getValue( DBlockTable.DIRECTIONS[ ( dir + 7 ) % 8 ] );
			if ( ! ( c0 || c1 ) || c0 && c1 && !c10 ) {
				return Block.FULL_BLOCK_AABB;
			}
		}
		return DBlockTable.COLLISION_TOP;
	}


	@Override
	public void addCollisionBoxToList( final IBlockState state , final World worldIn , final BlockPos pos ,
			final AxisAlignedBB container , final List< AxisAlignedBB > output , final Entity entity )
	{
		final IBlockState actual = this.getActualState( state , worldIn , pos );
		Block.addCollisionBoxToList( pos , container , output , DBlockTable.COLLISION_TOP );
		for ( int i = 0 , dir = 0 ; i < 4 ; i++ , dir += 2 ) {
			final boolean c0 = actual.getValue( DBlockTable.DIRECTIONS[ dir ] );
			final boolean c1 = actual.getValue( DBlockTable.DIRECTIONS[ ( dir + 6 ) % 8 ] );
			final boolean c10 = actual.getValue( DBlockTable.DIRECTIONS[ ( dir + 7 ) % 8 ] );
			if ( ! ( c0 || c1 ) || c0 && c1 && !c10 ) {
				Block.addCollisionBoxToList( pos , container , output , DBlockTable.COLLISION_LEGS[ i ] );
			}
		}
	}


	@Override
	@SideOnly( Side.CLIENT )
	public BlockRenderLayer getBlockLayer( )
	{
		return BlockRenderLayer.CUTOUT;
	}


	@Override
	public boolean isSideSolid( IBlockState base_state , IBlockAccess world , BlockPos pos , EnumFacing side )
	{
		return side == EnumFacing.UP;
	}
}
